use std::sync::Arc;

use bevy::prelude::*;
use bevy::tasks::futures_lite::{future, FutureExt};
use bevy::tasks::{block_on, AsyncComputeTaskPool, IoTaskPool, Task};
use voronoice::Voronoi;

use crate::map::{CellsEntities, Chunk, MeshNeedsUpdate};

use super::Cell;

pub struct Plugin;
impl bevy::prelude::Plugin for Plugin {
    fn build(&self, app: &mut App) {}
}

// Background task that manages terrain
#[derive(Resource)]
pub struct GrowThread(pub Task<()>);

#[derive(Component)]
struct GetChunkTask(Task<Vec<Cell>>);

fn start_grow_task(mut cmds: Commands) {}

fn rx_chunks_updates(
    mut chunks: Query<(&Chunk, &mut GetChunkTask, &mut MeshNeedsUpdate)>,
    mut cells: Query<&mut Cell>,
    cells_entities: Res<CellsEntities>,
) {
    for (ch_id, mut task, mut needs_update) in chunks.iter_mut() {
        let res = block_on(future::poll_once(&mut task.0));
        if let Some(updated_cells) = res {
            if updated_cells.len() > 0 {
                needs_update.0 = true;
            }
            for c in updated_cells.into_iter() {
                let id = c.voronoi_id;
                *cells.get_mut(cells_entities.0[id]).unwrap() = c;
            }
            let task_pool = IoTaskPool::get();
            task.0 = task_pool.spawn(async { Vec::new() })
        }
    }
}

pub async fn grow_thread(voronoi: Voronoi, current_frame: Arc<AtomicU>) {}

// fn begin_generating_map_chunks(mut my_tasks: ResMut<MyMapGenTasks>) {
//     let task_pool = AsyncComputeTaskPool::get();
//     for chunk_coord in decide_what_chunks_to_generate(/* ... */) {
//         // we might have already spawned a task for this `chunk_coord`
//         if my_tasks.generating_chunks.contains_key(&chunk_coord) {
//             continue;
//         }
//         let task = task_pool.spawn(async move {
//             // TODO: do whatever you want here!
//             generate_map_chunk(chunk_coord)
//         });
//         my_tasks.generating_chunks.insert(chunk_coord, task);
//     }
// }

// fn receive_generated_map_chunks(mut my_tasks: ResMut<MyMapGenTasks>) {
//     my_tasks.generating_chunks.retain(|chunk_coord, task| {
//         // check on our task to see how it's doing :)
//         let status = block_on(future::poll_once(task));

//         // keep the entry in our HashMap only if the task is not done yet
//         let retain = status.is_none();

//         // if this task is done, handle the data it returned!
//         if let Some(mut chunk_data) = status {
//             // TODO: do something with the returned `chunk_data`
//         }

//         retain
//     });
// }
