use std::time::Duration;

use bevy::prelude::*;

use crate::{
    map::{animals::Animal, MeshMarker, MeshNeedsUpdate, Voronoi},
    time::GameTime,
    ui::CurrentAction,
};

use super::{Cell, CellKind, WealthType};

pub fn on_click(
    trigger: Trigger<Pointer<Click>>,
    mut cells: Query<(&mut Cell, &Parent)>,
    voronoi: Res<Voronoi>,
    mut chunks: Query<&mut MeshNeedsUpdate, With<MeshMarker>>,
    mut cmds: Commands,
    ca: Res<CurrentAction>,
    gt: Res<GameTime>,
) {
    if trigger.duration > Duration::from_millis(200) {
        return;
    }
    let (mut cell, parent) = cells.get_mut(trigger.target).unwrap();
    match *ca {
        CurrentAction::ChangeCell(ck) => match ck {
            CellKind::Forest { .. } => match cell.kind {
                CellKind::Dirt | CellKind::Grass { .. } => {
                    cell.kind = CellKind::Forest {
                        wealth: WealthType::MAX / 2,
                    };
                    chunks.get_mut(parent.get()).unwrap().0 = true;
                }
                _ => {}
            },
            CellKind::Grass { .. } => match cell.kind {
                CellKind::Dirt => {
                    cell.kind = CellKind::Grass {
                        wealth: WealthType::MAX / 2,
                    };
                    chunks.get_mut(parent.get()).unwrap().0 = true;
                }
                _ => {}
            },
            _ => {}
        },
        CurrentAction::AddAnimal(ak) => {
            if cell.kind.can_place_animal(ak) {
                let v_cell = voronoi.0.cell(cell.voronoi_id);
                let cell_pos = v_cell.site_position();
                cmds.spawn((
                    Animal { kind: ak },
                    Transform::from_xyz(cell_pos.x as f32, cell_pos.y as f32, 0.),
                ));
            }
        }
        _ => {}
    }
}
